__author__ = 'LucasGonc'
__copyright__ = 'Copyright (c) 2024 Lucas Gonçalves da Silva'
__credits__ = ['LucasGonc']
__license__ = 'MIT'
__maintainer__ = 'LucasGonc'
__email__ = 'lucasgoncsilva04@gmail.com'
__repository__ = 'https://github.com/LucasGoncSilva/mosheh'
__keywords__ = ['CLI', 'Python', 'documentation', 'MkDocs', 'automation', 'generation']
__version__ = '1.3.2'
__date__ = '2024-12-17'
__status__ = 'Production'
